@@ -15,12 +15,12 @@
 #include "st.h"
 
 /* Private data used by writer */
-struct svxpriv {
+typedef struct svxpriv {
 	ULONG nsamples;
 	FILE *ch[4];
-};
+}*svx_t;
 
-static void svxwriteheader(P2(ft_t, LONG));
+static void svxwriteheader(ft_t, LONG);
 
 /*======================================================================*/
 /*                         8SVXSTARTREAD                                */
@@ -29,7 +29,7 @@ static void svxwriteheader(P2(ft_t, LONG));
 int st_svxstartread(ft)
 ft_t ft;
 {
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 
 	char buf[12];
 	char *chunk_buf;
@@ -43,6 +43,11 @@ ft_t ft;
 
 	ULONG chan1_pos;
 
+	if (! ft->seekable)
+	{
+		st_fail_errno(ft,ST_EINVAL,"8svx input file must be a file, not a pipe");
+		return (ST_EOF);
+	}
 	/* 8svx is in big endian format. Swap whats
 	 * read in on little endian machines.
 	 */
@@ -172,6 +177,7 @@ ft_t ft;
 	}
 	st_readdw(ft, &(p->nsamples));
 
+	ft->length = p->nsamples;
 	ft->info.channels = channels;
 	ft->info.rate = rate;
 	ft->info.encoding = ST_ENCODING_SIGN2;
@@ -215,7 +221,7 @@ LONG *buf, nsamp;
 	int done = 0;
 	int i;
 
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 
 	while (done < nsamp) {
 		for (i = 0; i < ft->info.channels; i++) {
@@ -239,7 +245,7 @@ ft_t ft;
 {
 	int i;
 
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 
 	/* close channel files */
 	for (i = 1; i < ft->info.channels; i++) {
@@ -254,7 +260,7 @@ ft_t ft;
 int st_svxstartwrite(ft)
 ft_t ft;
 {
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 	int i;
 
 	/* 8svx is in big endian format.  Swaps wahst
@@ -292,7 +298,7 @@ LONG st_svxwrite(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 
 	unsigned char datum;
 	int done = 0;
@@ -318,7 +324,7 @@ LONG *buf, len;
 int st_svxstopwrite(ft)
 ft_t ft;
 {
-	struct svxpriv *p = (struct svxpriv *) ft->priv;
+	svx_t p = (svx_t ) ft->priv;
 
 	int i;
 	int len;
