@@ -53,24 +53,40 @@
 #include "st.h"
 
 /* Private data used by writer */
-struct aiffpriv {
+typedef struct aiffpriv {
 	ULONG nsamples;	/* number of 1-channel samples read or written */
-};
+					/*Decrements for read increments for write */
+    LONG dataStart;  /* need to for seeking */
+} *aiff_t;
 
 /* forward declarations */
-static double read_ieee_extended(P1(ft_t));
-static int aiffwriteheader(P2(ft_t, LONG));
-static void write_ieee_extended(P2(ft_t, double));
-static double ConvertFromIeeeExtended(P1(unsigned char*));
-static void ConvertToIeeeExtended(P2(double, char *));
-static int textChunk(P3(char **text, char *chunkDescription, ft_t ft));
-static int commentChunk(P3(char **text, char *chunkDescription, ft_t ft));
-static void reportInstrument(P1(ft_t ft));
+static double read_ieee_extended(ft_t);
+static int aiffwriteheader(ft_t, LONG);
+static void write_ieee_extended(ft_t, double);
+static double ConvertFromIeeeExtended(unsigned char*);
+static void ConvertToIeeeExtended(double, char *);
+static int textChunk(char **text, char *chunkDescription, ft_t ft);
+static int commentChunk(char **text, char *chunkDescription, ft_t ft);
+static void reportInstrument(ft_t ft);
+
+int st_aiffseek(ft,offset) 
+ft_t ft;
+LONG offset;
+{
+	aiff_t aiff = (aiff_t ) ft->priv;
+
+	ft->st_errno = st_seek(ft,offset*ft->info.size + aiff->dataStart,SEEK_SET);
+
+	if( ft->st_errno == ST_SUCCESS )
+		aiff->nsamples = ft->length - offset;
+
+	return(ft->st_errno);
+}
 
 int st_aiffstartread(ft) 
 ft_t ft;
 {
-	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+	aiff_t aiff = (aiff_t ) ft->priv;
 	char buf[5];
 	ULONG totalsize;
 	LONG chunksize;
@@ -100,11 +116,6 @@ ft_t ft;
 
 	int rc;
 
-	/* Needed because of st_rawread() */
-	rc = st_rawstartread(ft);
-	if (rc)
-	    return rc;
-
 	/* AIFF is in Big Endian format.  Swap whats read in on Little */
 	/* Endian machines.                                            */
 	if (ST_IS_LITTLEENDIAN)
@@ -115,14 +126,14 @@ ft_t ft;
 	/* FORM chunk */
 	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
 	{
-		st_fail("AIFF header does not begin with magic word 'FORM'");
+		st_fail_errno(ft,ST_EHDR,"AIFF header does not begin with magic word 'FORM'");
 		return(ST_EOF);
 	}
 	st_readdw(ft, &totalsize);
 	if (st_reads(ft, buf, 4) == ST_EOF || (strncmp(buf, "AIFF", 4) != 0 && 
 	    strncmp(buf, "AIFC", 4) != 0))
 	{
-		st_fail("AIFF 'FORM' chunk does not specify 'AIFF' or 'AIFC' as type");
+		st_fail_errno(ft,ST_EHDR,"AIFF 'FORM' chunk does not specify 'AIFF' or 'AIFC' as type");
 		return(ST_EOF);
 	}
 
@@ -136,7 +147,7 @@ ft_t ft;
 				break;
 			else
 			{
-				st_fail("Missing SSND chunk in AIFF file");
+				st_fail_errno(ft,ST_EHDR,"Missing SSND chunk in AIFF file");
 				return(ST_EOF);
 			}
 		}
@@ -155,7 +166,7 @@ ft_t ft;
 			    if (strncmp(buf, "NONE", 4) != 0)
 			    {
 				buf[4] = 0;
-				st_fail("Can not support AIFC files that contain compressed data: %s",buf);
+				st_fail_errno(ft,ST_EHDR,"Can not support AIFC files that contain compressed data: %s",buf);
 				return(ST_EOF);
 			    }
 			}
@@ -235,15 +246,7 @@ ft_t ft;
 			st_readw(ft, &releaseLoopBegin);  /* begin marker */
 			st_readw(ft, &releaseLoopEnd);    /* end marker */
 
-			/* Required to ignore loops on playback if type
-			 * is 0 (NoLoop).  The other check is done for
-			 * historical reason and is probably not needed.
-			 */
-			if ((ft->loops[0].type == 0) ||
-			   (sustainLoopBegin == 0 && releaseLoopBegin == 0))
-				foundinstr = 0;
-			else
-				foundinstr = 1;
+			foundinstr = 1;
 		}
 		else if (strncmp(buf, "APPL", 4) == 0) {
 			st_readdw(ft, &chunksize);
@@ -338,20 +341,20 @@ ft_t ft;
 			fseek(ft->fp, seekto, SEEK_SET);
 		else
 		{
-			st_fail("AIFF: no sound data on input file");
+			st_fail_errno(ft,ST_EOF,"AIFF: no sound data on input file");
 			return(ST_EOF);
 		}
 	}
 	/* SSND chunk just read */
 	if (blocksize != 0)
 	{
-		st_fail("AIFF header specifies nonzero blocksize?!?!");
+		st_fail_errno(ft,ST_EHDR,"AIFF header specifies nonzero blocksize?!?!");
 		return(ST_EOF);
 	}
 	while ((LONG) (--offset) >= 0) {
 		if (st_readb(ft, (unsigned char *)&trash) == ST_EOF)
 		{
-			st_fail("unexpected EOF while skipping AIFF offset");
+			st_fail_errno(ft,errno,"unexpected EOF while skipping AIFF offset");
 			return(ST_EOF);
 		}
 	}
@@ -360,17 +363,28 @@ ft_t ft;
 		ft->info.channels = channels;
 		ft->info.rate = rate;
 		ft->info.encoding = ST_ENCODING_SIGN2;
-		switch (bits) {
-		case 8:
-			ft->info.size = ST_SIZE_BYTE;
-			break;
-		case 16:
-			ft->info.size = ST_SIZE_WORD;
-			break;
-		default:
-			st_fail("unsupported sample size in AIFF header: %d", bits);
-			return(ST_EOF);
-			/*NOTREACHED*/
+		if (bits <= 8)
+		{
+		    ft->info.size = ST_SIZE_BYTE;
+		    if (bits < 8)
+			st_report("Forcing data size from %d bits to 8 bits",bits);
+		}
+		else if (bits <= 16)
+		{
+		    ft->info.size = ST_SIZE_WORD;
+		    if (bits < 16)
+			st_report("Forcing data size from %d bits to 16 bits",bits);
+		}
+		else if (bits <= 32)
+		{
+		    ft->info.size = ST_SIZE_DWORD;
+		    if (bits < 32)
+			st_report("Forcing data size from %d bits to 32 bits",bits);
+		}
+		else
+		{
+		    st_fail_errno(ft,ST_EFMT,"unsupported sample size in AIFF header: %d", bits);
+		    return(ST_EOF);
 		}
 	} else  {
 		if ((ft->info.channels == -1)
@@ -379,23 +393,23 @@ ft_t ft;
 			|| (ft->info.size == -1)) {
 		  st_report("You must specify # channels, sample rate, signed/unsigned,\n");
 		  st_report("and 8/16 on the command line.");
-		  st_fail("Bogus AIFF file: no COMM section.");
+		  st_fail_errno(ft,ST_EFMT,"Bogus AIFF file: no COMM section.");
 		  return(ST_EOF);
 		}
 
 	}
 
-	p->nsamples = ssndsize / ft->info.size;	/* leave out channels */
+	aiff->nsamples = ssndsize / ft->info.size;	/* leave out channels */
 
 	if (foundmark && !foundinstr)
 	{
-		st_fail("Bogus AIFF file: MARKers but no INSTrument.");
-		return(ST_EOF);
+	    st_report("Ignoring MARK chunk since no INSTR found.");
+	    foundmark = 0;
 	}
 	if (!foundmark && foundinstr)
 	{
-		st_fail("Bogus AIFF file: INSTrument but no MARKers.");
-		return(ST_EOF);
+	    st_report("Ignoring INSTR chunk since no MARK found.");
+	    foundinstr = 0;
 	}
 	if (foundmark && foundinstr) {
 		int i;
@@ -434,8 +448,15 @@ ft_t ft;
 			ft->instr.nloops++;
 		} 
 	}
-	if (verbose)
-	  reportInstrument(ft);
+	reportInstrument(ft);
+
+	/* Needed because of st_rawread() */
+	rc = st_rawstartread(ft);
+	if (rc)
+	    return rc;
+
+	ft->length = aiff->nsamples; 	/* for seeking */
+	aiff->dataStart = ftell(ft->fp);
 
 	return(ST_SUCCESS);
 }
@@ -447,24 +468,24 @@ ft_t ft;
   int loopNum;
 
   if(ft->instr.nloops > 0)
-    fprintf(stderr, "AIFF Loop markers:\n");
+    st_report("AIFF Loop markers:\n");
   for(loopNum  = 0; loopNum < ft->instr.nloops; loopNum++) {
     if (ft->loops[loopNum].count) {
-      fprintf(stderr, "Loop %d: start: %6d", loopNum, ft->loops[loopNum].start);
-      fprintf(stderr, " end:   %6d", 
+      st_report("Loop %d: start: %6d", loopNum, ft->loops[loopNum].start);
+      st_report(" end:   %6d", 
 	      ft->loops[loopNum].start + ft->loops[loopNum].length);
-      fprintf(stderr, " count: %6d", ft->loops[loopNum].count);
-      fprintf(stderr, " type:  ");
+      st_report(" count: %6d", ft->loops[loopNum].count);
+      st_report(" type:  ");
       switch(ft->loops[loopNum].type & ~ST_LOOP_SUSTAIN_DECAY) {
-      case 0: fprintf(stderr, "off\n"); break;
-      case 1: fprintf(stderr, "forward\n"); break;
-      case 2: fprintf(stderr, "forward/backward\n"); break;
+      case 0: st_report("off\n"); break;
+      case 1: st_report("forward\n"); break;
+      case 2: st_report("forward/backward\n"); break;
       }
     }
   }
-  fprintf(stderr, "Unity MIDI Note: %d\n", ft->instr.MIDInote);
-  fprintf(stderr, "Low   MIDI Note: %d\n", ft->instr.MIDIlow);
-  fprintf(stderr, "High  MIDI Note: %d\n", ft->instr.MIDIhi);
+  st_report("Unity MIDI Note: %d\n", ft->instr.MIDInote);
+  st_report("Low   MIDI Note: %d\n", ft->instr.MIDIlow);
+  st_report("High  MIDI Note: %d\n", ft->instr.MIDIhi);
 }
 
 /* Process a text chunk, allocate memory, display it if verbose and return */
@@ -479,12 +500,12 @@ ft_t ft;
   *text = (char *) malloc((size_t) chunksize + 1);
   if (*text == NULL)
   {
-    st_fail("AIFF: Couldn't allocate %s header", chunkDescription);
+    st_fail_errno(ft,ST_ENOMEM,"AIFF: Couldn't allocate %s header", chunkDescription);
     return(ST_EOF);
   }
   if (fread(*text, 1, chunksize, ft->fp) != chunksize)
   {
-    st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+    st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
     return(ST_EOF);
   }
   *(*text + chunksize) = '\0';
@@ -494,13 +515,11 @@ ft_t ft;
 		char c;
 		if (fread(&c, 1, 1, ft->fp) != 1)
 		{
-			st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+    		st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
 			return(ST_EOF);
 		}
 	}
-  if(verbose) {
-    printf("%-10s   \"%s\"\n", chunkDescription, *text);
-  }
+  st_report("%-10s   \"%s\"\n", chunkDescription, *text);
   return(ST_SUCCESS);
 }
 
@@ -537,11 +556,11 @@ ft_t ft;
     }
 
     if (*text == NULL) {
-	st_fail("AIFF: Couldn't allocate %s header", chunkDescription);
+	st_fail_errno(ft,ST_ENOMEM,"AIFF: Couldn't allocate %s header", chunkDescription);
 	return(ST_EOF);
     }
     if (fread(*text + totalCommentLength - commentLength, 1, commentLength, ft->fp) != commentLength) {
-	st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+	st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
 	return(ST_EOF);
     }
     *(*text + totalCommentLength) = '\0';
@@ -549,14 +568,12 @@ ft_t ft;
 	/* Read past pad byte */
 	char c;
 	if (fread(&c, 1, 1, ft->fp) != 1) {
-	    st_fail("AIFF: Unexpected EOF in %s header", chunkDescription);
+	    st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
 	    return(ST_EOF);
 	}
     }
   }
-  if(verbose) {
-    printf("%-10s   \"%s\"\n", chunkDescription, *text);
-  }
+  st_report("%-10s   \"%s\"\n", chunkDescription, *text);
   return(ST_SUCCESS);
 }
 
@@ -564,14 +581,17 @@ LONG st_aiffread(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
-	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+	aiff_t aiff = (aiff_t ) ft->priv;
+	LONG done;
 
 	/* just read what's left of SSND chunk */
-	if (len > p->nsamples)
-		len = p->nsamples;
-	st_rawread(ft, buf, len);
-	p->nsamples -= len;
-	return len;
+	if (len > aiff->nsamples)
+		len = aiff->nsamples;
+	done = st_rawread(ft, buf, len);
+	if (done == 0 && aiff->nsamples != 0)
+		st_warn("Premature EOF on AIFF input file");
+	aiff->nsamples -= done;
+	return done;
 }
 
 int st_aiffstopread(ft) 
@@ -621,7 +641,7 @@ ft_t ft;
 int st_aiffstartwrite(ft)
 ft_t ft;
 {
-	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+	aiff_t aiff = (aiff_t ) ft->priv;
 	int rc;
 
 	/* Needed because st_rawwrite() */
@@ -636,7 +656,7 @@ ft_t ft;
 	    ft->swap = ft->swap ? 0 : 1;
 	}
 
-	p->nsamples = 0;
+	aiff->nsamples = 0;
 	if ((ft->info.encoding == ST_ENCODING_ULAW ||
 	     ft->info.encoding == ST_ENCODING_ALAW) && 
 	    ft->info.size == ST_SIZE_BYTE) {
@@ -658,8 +678,8 @@ LONG st_aiffwrite(ft, buf, len)
 ft_t ft;
 LONG *buf, len;
 {
-	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
-	p->nsamples += len;
+	aiff_t aiff = (aiff_t ) ft->priv;
+	aiff->nsamples += len;
 	st_rawwrite(ft, buf, len);
 	return(len);
 }
@@ -667,7 +687,7 @@ LONG *buf, len;
 int st_aiffstopwrite(ft)
 ft_t ft;
 {
-	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+	aiff_t aiff = (aiff_t ) ft->priv;
 	int rc;
 
 	/* Needed because of st_rawwrite().  Call now to flush
@@ -679,15 +699,15 @@ ft_t ft;
 
 	if (!ft->seekable)
 	{
-	    st_fail("Non-seekable file.");
+	    st_fail_errno(ft,ST_EOF,"Non-seekable file.");
 	    return(ST_EOF);
 	}
 	if (fseek(ft->fp, 0L, SEEK_SET) != 0)
 	{
-		st_fail("can't rewind output file to rewrite AIFF header");
+		st_fail_errno(ft,errno,"can't rewind output file to rewrite AIFF header");
 		return(ST_EOF);
 	}
-	return(aiffwriteheader(ft, p->nsamples / ft->info.channels));
+	return(aiffwriteheader(ft, aiff->nsamples / ft->info.channels));
 }
 
 static int aiffwriteheader(ft, nframes)
@@ -717,7 +737,7 @@ LONG nframes;
 		bits = 16;
 	else
 	{
-		st_fail("unsupported output encoding/size for AIFF header");
+		st_fail_errno(ft,ST_EFMT,"unsupported output encoding/size for AIFF header");
 		return(ST_EOF);
 	}
 
@@ -834,7 +854,7 @@ ft_t ft;
 	char buf[10];
 	if (fread(buf, 1, 10, ft->fp) != 10)
 	{
-		st_fail("EOF while reading IEEE extended number");
+		st_fail_errno(ft,ST_EOF,"EOF while reading IEEE extended number");
 		return(ST_EOF);
 	}
 	return ConvertFromIeeeExtended(buf);
