@@ -27,6 +27,7 @@ extern int  st_aiffstopread();
 extern int  st_aiffstartwrite();
 extern LONG st_aiffwrite();
 extern int  st_aiffstopwrite();
+extern int  st_aiffseek();
 
 static char *alnames[] = {
 	"al",
@@ -57,6 +58,7 @@ extern LONG st_auread();
 extern int  st_austartwrite();
 extern LONG st_auwrite();
 extern int  st_austopwrite();
+extern int  st_auseek();
 
 static char *autonames[] = {
 	"auto",
@@ -159,6 +161,18 @@ extern LONG st_maudwrite();
 extern int  st_maudstartwrite();
 extern int  st_maudstopwrite();
 
+static char *nulnames[] = {
+        "nul",
+        (char *) 0,
+};
+
+extern int  st_nulstartread();
+extern LONG st_nulread();
+extern int  st_nulstopread();
+extern LONG st_nulwrite();
+extern int  st_nulstartwrite();
+extern int  st_nulstopwrite();
+
 #if	defined(OSS_PLAYER)
 static char *ossdspnames[] = {
 	"ossdsp",
@@ -191,6 +205,7 @@ static char *sfnames[] = {
 
 extern int st_sfstartread();
 extern int st_sfstartwrite();
+extern int st_sfseek();
 
 static char *slnames[] = {
 	"sl",
@@ -210,6 +225,7 @@ extern LONG st_smpread();
 extern LONG st_smpwrite();
 extern int  st_smpstartwrite();
 extern int  st_smpstopwrite();
+extern int  st_smpseek();
 
 static char *sndtnames[] = {
 	"sndt",
@@ -223,6 +239,7 @@ extern int  st_sndtstartread();
 extern int  st_sndtstartwrite();
 extern LONG st_sndtwrite();
 extern int  st_sndtstopwrite();
+extern int  st_sndseek();
 
 static char *spherenames[] = {
 	"sph",
@@ -318,6 +335,21 @@ extern int  st_vocstartwrite();
 extern LONG st_vocwrite();
 extern int  st_vocstopwrite();
 
+#ifdef HAVE_LIBVORBIS
+static char *vorbisnames[] = {
+	"vorbis",
+	"ogg",
+	(char *) 0
+};
+
+extern int  st_vorbisstartread();
+extern LONG st_vorbisread();
+extern int  st_vorbisstopread();
+extern int  st_vorbisstartwrite();
+extern LONG st_vorbiswrite();
+extern int  st_vorbisstopwrite();
+#endif
+
 static char *wavnames[] = {
 	"wav",
 	(char *) 0
@@ -328,6 +360,7 @@ extern LONG st_wavread();
 extern int  st_wavstartwrite();
 extern LONG st_wavwrite();
 extern int  st_wavstopwrite();
+extern int  st_wavseek();
 
 static char *wvenames[] = {
       "wve",
@@ -339,112 +372,123 @@ extern LONG st_wveread();
 extern int  st_wvestartwrite();
 extern LONG st_wvewrite();
 extern int  st_wvestopwrite();
+extern int  st_wveseek();
 
 extern int  st_nothing();
 extern LONG st_nothing_success();
 
 st_format_t st_formats[] = {
-	{aiffnames, ST_FILE_STEREO,		/* SGI/Apple AIFF */
-		st_aiffstartread, st_aiffread, st_aiffstopread,
-		st_aiffstartwrite, st_aiffwrite, st_aiffstopwrite},
+	{aiffnames, 					/* SGI/Apple AIFF */
+
+	    ST_FILE_STEREO | ST_FILE_LOOPS | ST_FILE_SEEK,
+	        st_aiffstartread, st_aiffread, st_aiffstopread,
+		st_aiffstartwrite, st_aiffwrite, st_aiffstopwrite, st_aiffseek},
 	{alnames, ST_FILE_STEREO,		/* a-law byte raw */
 		st_alstartread, st_rawread, st_rawstopread,
-		st_alstartwrite, st_rawwrite, st_rawstopwrite},	
+		st_alstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},	
 #if	defined(ALSA_PLAYER)
 	{alsanames, ST_FILE_STEREO,		/* /dev/snd/pcmXX */
 		st_alsastartread, st_rawread, st_rawstopread,
-		st_alsastartwrite, st_rawwrite, st_rawstopwrite},
+		st_alsastartwrite, st_rawwrite, st_rawstopwrite, st_nothing},
 #endif
-	{aunames, ST_FILE_STEREO,		/* SPARC .au w/header */
+	{aunames, ST_FILE_STEREO | ST_FILE_SEEK,		/* SPARC .au w/header */
 		st_austartread, st_auread, st_rawstopread,
-		st_austartwrite, st_auwrite, st_austopwrite},	
+		st_austartwrite, st_auwrite, st_austopwrite, st_auseek},	
 	{autonames, ST_FILE_STEREO,		/* Guess from header */
 		st_autostartread, st_nothing_success, st_nothing,
-		st_autostartwrite, st_nothing_success, st_nothing},
+		st_autostartwrite, st_nothing_success, st_nothing, st_nothing},
 	{avrnames, ST_FILE_STEREO,		/* AVR format */
 		st_avrstartread, st_rawread, st_nothing,	
-		st_avrstartwrite, st_avrwrite, st_avrstopwrite},
-	{cdrnames, ST_FILE_STEREO,		/* CD-R format */
+		st_avrstartwrite, st_avrwrite, st_avrstopwrite, st_nothing},
+	{cdrnames, ST_FILE_STEREO | ST_FILE_SEEK,		/* CD-R format */
 		st_cdrstartread, st_cdrread, st_cdrstopread,
-		st_cdrstartwrite, st_cdrwrite, st_cdrstopwrite},
+		st_cdrstartwrite, st_cdrwrite, st_cdrstopwrite, st_rawseek},
 	{cvsdnames, 0,			/* Cont. Variable Slope Delta */
 	        st_cvsdstartread, st_cvsdread, st_cvsdstopread,
-	        st_cvsdstartwrite, st_cvsdwrite, st_cvsdstopwrite},
+	        st_cvsdstartwrite, st_cvsdwrite, st_cvsdstopwrite, st_nothing},
 	{datnames, 0,				/* Text data samples */
 		st_datstartread, st_datread, st_nothing,
-		st_datstartwrite, st_datwrite, st_nothing},
+		st_datstartwrite, st_datwrite, st_nothing, st_nothing},
 	{dvmsnames, 0,			/* Cont. Variable Solot Delta */
 	        st_dvmsstartread, st_cvsdread, st_cvsdstopread,
-	        st_dvmsstartwrite, st_cvsdwrite, st_dvmsstopwrite},
+	        st_dvmsstartwrite, st_cvsdwrite, st_dvmsstopwrite, st_nothing},
 #ifdef HAVE_LIBGSM
 	{gsmnames, 0,				/* GSM 06.10 */
 	        st_gsmstartread, st_gsmread, st_gsmstopread,
-	        st_gsmstartwrite, st_gsmwrite, st_gsmstopwrite},
+	        st_gsmstartwrite, st_gsmwrite, st_gsmstopwrite, st_nothing},
 #endif
 	{hcomnames, 0,				/* Mac FSSD/HCOM */
 		st_hcomstartread, st_hcomread, st_hcomstopread, 
-		st_hcomstartwrite, st_hcomwrite, st_hcomstopwrite},
+		st_hcomstartwrite, st_hcomwrite, st_hcomstopwrite, st_nothing},
         {maudnames, ST_FILE_STEREO,    		/* Amiga MAUD */
 		st_maudstartread, st_maudread, st_maudstopread,
-		st_maudstartwrite, st_maudwrite, st_maudstopwrite},
+		st_maudstartwrite, st_maudwrite, st_maudstopwrite, st_nothing},
+        {nulnames, ST_FILE_STEREO,    		/* NUL */
+ 		st_nulstartread, st_nulread, st_nulstopread,
+ 		st_nulstartwrite, st_nulwrite, st_nulstopwrite},
 #if	defined(OSS_PLAYER)
 	{ossdspnames, ST_FILE_STEREO,		/* OSS /dev/dsp player */
 		st_ossdspstartread, st_rawread, st_rawstopread,
-		st_ossdspstartwrite, st_rawwrite, st_rawstopwrite},
+		st_ossdspstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},
 #endif
-	{rawnames, ST_FILE_STEREO,		/* Raw format */
+	{rawnames, ST_FILE_STEREO | ST_FILE_SEEK,		/* Raw format */
 		st_rawstartread, st_rawread, st_rawstopread,
-		st_rawstartwrite, st_rawwrite, st_rawstopwrite},
+		st_rawstartwrite, st_rawwrite, st_rawstopwrite, st_rawseek},
 	{sbnames, ST_FILE_STEREO,		/* signed byte raw */
 		st_sbstartread, st_rawread, st_rawstopread,
-		st_sbstartwrite, st_rawwrite, st_rawstopwrite},	
-	{sfnames, ST_FILE_STEREO,		/* IRCAM Sound File */
+		st_sbstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},	
+	{sfnames, ST_FILE_STEREO | ST_FILE_SEEK,		/* IRCAM Sound File */
 		st_sfstartread, st_rawread, st_rawstopread,
-		st_sfstartwrite, st_rawwrite, st_rawstopwrite},
+		st_sfstartwrite, st_rawwrite, st_rawstopwrite, st_sfseek},
 	{ slnames, ST_FILE_STEREO,		/* signed long raw */
 	    	st_slstartread, st_rawread, st_rawstopread,
-		st_slstartwrite, st_rawwrite, st_rawstopwrite },
-	{smpnames, ST_FILE_STEREO | ST_FILE_LOOPS,/* SampleVision sound */
+		st_slstartwrite, st_rawwrite, st_rawstopwrite, st_nothing },
+	{smpnames, ST_FILE_STEREO | ST_FILE_LOOPS | ST_FILE_SEEK,/* SampleVision sound */
 		st_smpstartread, st_smpread, st_nothing,
-		st_smpstartwrite, st_smpwrite, st_smpstopwrite},
-	{sndtnames, ST_FILE_STEREO,		/* Sndtool Sound File */
+		st_smpstartwrite, st_smpwrite, st_smpstopwrite, st_smpseek},
+	{sndtnames, ST_FILE_STEREO | ST_FILE_SEEK,		/* Sndtool Sound File */
 		st_sndtstartread, st_rawread, st_rawstopread, 
-		st_sndtstartwrite, st_sndtwrite, st_sndtstopwrite},
+		st_sndtstartwrite, st_sndtwrite, st_sndtstopwrite, st_sndseek},
 	{spherenames, ST_FILE_STEREO,		/* NIST Sphere File */
 	        st_spherestartread, st_sphereread, st_rawstopread,
-		st_spherestartwrite, st_spherewrite, st_spherestopwrite},
+		st_spherestartwrite, st_spherewrite, st_spherestopwrite, st_nothing},
 #if	defined(SUNAUDIO_PLAYER)
 	{sunnames, ST_FILE_STEREO,		/* Sun /dev/audio player */
 		st_sunstartread, st_rawread, st_rawstopread,
-		st_sunstartwrite, st_rawwrite, st_rawstopwrite},
+		st_sunstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},
 #endif
 	{svxnames, ST_FILE_STEREO,		/* Amiga 8SVX */
 		st_svxstartread, st_svxread, st_svxstopread,
-		st_svxstartwrite, st_svxwrite, st_svxstopwrite},
+		st_svxstartwrite, st_svxwrite, st_svxstopwrite, st_nothing},
 	{swnames, ST_FILE_STEREO,		/* signed word raw */
 		st_swstartread, st_rawread, st_rawstopread,
-		st_swstartwrite, st_rawwrite, st_rawstopwrite},
+		st_swstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},
 	{txwnames, 0,			/* Yamaha TX16W and SY99 waves */
 	        st_txwstartread, st_txwread, st_txwstopread, 
-	        st_txwstartwrite, st_txwwrite, st_txwstopwrite},
+	        st_txwstartwrite, st_txwwrite, st_txwstopwrite, st_nothing},
 	{ubnames, ST_FILE_STEREO,		/* unsigned byte raw */
 		st_ubstartread, st_rawread, st_rawstopread,
-		st_ubstartwrite, st_rawwrite, st_rawstopwrite},
+		st_ubstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},
 	{ulnames, ST_FILE_STEREO,		/* u-law byte raw */
 		st_ulstartread, st_rawread, st_rawstopread,
-		st_ulstartwrite, st_rawwrite, st_rawstopwrite},	
+		st_ulstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},	
 	{uwnames, ST_FILE_STEREO,		/* unsigned word raw */
 		st_uwstartread, st_rawread, st_rawstopread,
-		st_uwstartwrite, st_rawwrite, st_rawstopwrite},	
+		st_uwstartwrite, st_rawwrite, st_rawstopwrite, st_nothing},	
 	{vocnames, ST_FILE_STEREO,		/* Sound Blaster .VOC */
 		st_vocstartread, st_vocread, st_vocstopread,
-		st_vocstartwrite, st_vocwrite, st_vocstopwrite},
-	{wavnames, ST_FILE_STEREO,		/* Microsoftt RIFF */
+		st_vocstartwrite, st_vocwrite, st_vocstopwrite, st_nothing},
+#ifdef HAVE_LIBVORBIS
+	{vorbisnames, ST_FILE_STEREO,		/* Ogg Vorbis */
+		st_vorbisstartread, st_vorbisread, st_vorbisstopread,
+		st_vorbisstartwrite, st_vorbiswrite, st_vorbisstopwrite},
+#endif
+	{wavnames, ST_FILE_STEREO | ST_FILE_SEEK,		/* Microsoftt RIFF */
 		st_wavstartread, st_wavread, st_nothing,
-		st_wavstartwrite, st_wavwrite, st_wavstopwrite},	
-	{wvenames, 0,				/* Psion .wve */
+		st_wavstartwrite, st_wavwrite, st_wavstopwrite, st_wavseek},	
+	{wvenames, ST_FILE_SEEK,				/* Psion .wve */
 		st_wvestartread, st_wveread, st_rawstopread,
-		st_wvestartwrite, st_wvewrite, st_wvestopwrite},
+		st_wvestartwrite, st_wvewrite, st_wvestopwrite, st_wveseek},
 	{0, 0,
 	 0, 0, 0, 0, 0, 0}
 };
@@ -477,16 +521,18 @@ extern int st_chorus_stop();
 extern int st_compand_getopts();
 extern int st_compand_start();
 extern int st_compand_flow();
+extern int st_compand_drain();
+extern int st_compand_stop();
 
 extern int st_copy_getopts(); 
 extern int st_copy_start();
 extern int st_copy_flow();
 extern int st_copy_stop();
 
-extern int st_cut_getopts();
-extern int st_cut_start();
-extern int st_cut_flow();
-extern int st_cut_stop();
+extern int st_dcshift_getopts();
+extern int st_dcshift_start();
+extern int st_dcshift_flow();
+extern int st_dcshift_stop();
 
 extern int st_deemph_getopts();
 extern int st_deemph_start();
@@ -603,6 +649,12 @@ extern int st_reverse_flow();
 extern int st_reverse_drain();
 extern int st_reverse_stop();
 
+extern int st_silence_getopts();
+extern int st_silence_start();
+extern int st_silence_flow();
+extern int st_silence_drain();
+extern int st_silence_stop();
+
 extern int st_speed_getopts();
 extern int st_speed_start();
 extern int st_speed_flow();
@@ -632,6 +684,17 @@ extern int st_swap_flow();
 extern int st_swap_drain();
 extern int st_swap_stop();
 
+extern int st_synth_getopts(); 
+extern int st_synth_start();
+extern int st_synth_flow();
+extern int st_synth_drain();
+extern int st_synth_stop();
+
+extern int st_trim_getopts(); 
+extern int st_trim_start();
+extern int st_trim_flow();
+extern int st_trim_stop();
+
 extern int st_vibro_getopts();
 extern int st_vibro_start();
 extern int st_vibro_flow();
@@ -642,11 +705,6 @@ extern int st_vol_start();
 extern int st_vol_flow();
 extern int st_vol_stop();
 
-extern int st_trim_getopts(); 
-extern int st_trim_start();
-extern int st_trim_flow();
-extern int st_trim_stop();
-
 /*
  * ST_EFF_CHAN means that the number of channels can change.
  * ST_EFF_RATE means that the sample rate can change.
@@ -658,7 +716,7 @@ st_effect_t st_effects[] = {
 	{"null", 0, 			/* stand-in, never gets called */
 		st_nothing, st_nothing, st_nothing, 
 		st_null_drain, st_nothing},
-	{"avg", ST_EFF_MCHAN | ST_EFF_CHAN, 
+	{"avg", ST_EFF_CHAN, 
 		st_avg_getopts, st_avg_start, st_avg_flow, 
 		st_null_drain, st_avg_stop},
 	{"band", 0, 
@@ -675,13 +733,13 @@ st_effect_t st_effects[] = {
 	 	st_chorus_drain, st_chorus_stop},
 	{"compand", ST_EFF_MCHAN,
 	        st_compand_getopts, st_compand_start, st_compand_flow,
-		st_null_drain, st_nothing},
+		st_compand_drain, st_compand_stop},
 	{"copy", ST_EFF_MCHAN, 
 		st_copy_getopts, st_copy_start, st_copy_flow, 
 		st_null_drain, st_nothing},
-	{"cut", ST_EFF_MCHAN, 
-		st_cut_getopts, st_cut_start, st_cut_flow, 
-		st_null_drain, st_nothing},
+	{"dcshift", ST_EFF_MCHAN, 
+		st_dcshift_getopts, st_dcshift_start, st_dcshift_flow, 
+		st_null_drain, st_dcshift_stop},
 	{"deemph", ST_EFF_MCHAN,
 	        st_deemph_getopts, st_deemph_start, st_deemph_flow,
 	        st_null_drain, st_deemph_stop},
@@ -748,6 +806,9 @@ st_effect_t st_effects[] = {
 	{"reverse", 0, 
 		st_reverse_getopts, st_reverse_start, 
 		st_reverse_flow, st_reverse_drain, st_reverse_stop},
+	{"silence", 0, 
+		st_silence_getopts, st_silence_start, 
+		st_silence_flow, st_silence_drain, st_silence_stop},
 	{"speed", 0, 
 		st_speed_getopts, st_speed_start, 
 		st_speed_flow, st_speed_drain, st_speed_stop},
@@ -763,16 +824,18 @@ st_effect_t st_effects[] = {
 	{"swap", ST_EFF_MCHAN,
 		st_swap_getopts, st_swap_start, st_swap_flow, 
 		st_swap_drain, st_swap_stop},
+        {"synth", ST_EFF_MCHAN, 
+                st_synth_getopts, st_synth_start, st_synth_flow, 
+                st_synth_drain, st_synth_stop},
+        {"trim", ST_EFF_MCHAN, 
+                st_trim_getopts, st_trim_start, st_trim_flow, 
+                st_null_drain, st_nothing},
 	{"vibro", 0, 
 		st_vibro_getopts, st_vibro_start, st_vibro_flow, 
 		st_null_drain, st_nothing},
 	{"vol", ST_EFF_MCHAN, 
 		st_vol_getopts, st_vol_start, st_vol_flow, 
 		st_null_drain, st_vol_stop},
-        {"trim", ST_EFF_MCHAN, 
-                st_trim_getopts, st_trim_start, st_trim_flow, 
-                st_null_drain, st_nothing},
-
 	{0, 0, 0, 0, 0, 0, 0}
 };
 
