@@ -31,7 +31,7 @@ struct maudstuff { /* max. 100 bytes!!!! */
 	ULONG nsamples;
 };
 
-static void maudwriteheader(P1(ft_t));
+static void maudwriteheader(ft_t);
 
 /*
  * Do anything required before you start reading samples.
@@ -73,7 +73,7 @@ ft_t ft;
 	/* read FORM chunk */
 	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
 	{
-		st_fail("MAUD: header does not begin with magic word 'FORM'");
+		st_fail_errno(ft,ST_EHDR,"MAUD: header does not begin with magic word 'FORM'");
 		return (ST_EOF);
 	}
 	
@@ -81,7 +81,7 @@ ft_t ft;
 	
 	if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "MAUD", 4) != 0)
 	{
-		st_fail("MAUD: 'FORM' chunk does not specify 'MAUD' as type");
+		st_fail_errno(ft,ST_EHDR,"MAUD: 'FORM' chunk does not specify 'MAUD' as type");
 		return(ST_EOF);
 	}
 	
@@ -99,8 +99,8 @@ ft_t ft;
 			st_readdw(ft, &chunksize);
 			if (chunksize != 8*4) 
 			{
-			    st_fail("MAUD: MHDR chunk has bad size");
-			    return(0);
+			    st_fail_errno(ft,ST_EHDR,"MAUD: MHDR chunk has bad size");
+			    return(ST_EOF);
 			}
 			
 			/* fseek(ft->fp,12,SEEK_CUR); */
@@ -118,7 +118,7 @@ ft_t ft;
 			st_readw(ft, &denom);       /* clock devide           */
 			if (denom == 0) 
 			{
-			    st_fail("MAUD: frequency denominator == 0, failed");
+			    st_fail_errno(ft,ST_EHDR,"MAUD: frequency denominator == 0, failed");
 			    return (ST_EOF);
 			}
 			
@@ -133,14 +133,14 @@ ft_t ft;
 				ft->info.channels = 2;
 				break;
 			default:
-				st_fail("MAUD: unsupported number of channels in file");
+				st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels in file");
 				return (ST_EOF);
 			}
 			
 			st_readw(ft, &chaninf); /* number of channels (mono: 1, stereo: 2, ...) */
 			if (chaninf != ft->info.channels) 
 			{
-			    st_fail("MAUD: unsupported number of channels in file");
+				st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels in file");
 			    return(ST_EOF);
 			}
 			
@@ -168,7 +168,7 @@ ft_t ft;
 			}
 			else 
 			{
-				st_fail("MAUD: unsupported compression type detected");
+				st_fail_errno(ft,ST_EFMT,"MAUD: unsupported compression type detected");
 				return(ST_EOF);
 			}
 			
@@ -184,13 +184,13 @@ ft_t ft;
 			chunk_buf = (char *) malloc(chunksize + 1);
 			if (!chunk_buf)
 			{
-			    st_fail("Couldn't alloc resources");
+			    st_fail_errno(ft,ST_ENOMEM,"Couldn't alloc resources");
 			    return(ST_EOF);
 			}
 			if (fread(chunk_buf,1,(int)chunksize,ft->fp) 
 					!= chunksize)
 			{
-				st_fail("MAUD: Unexpected EOF in ANNO header");
+				st_fail_errno(ft,ST_EOF,"MAUD: Unexpected EOF in ANNO header");
 				return(ST_EOF);
 			}
 			chunk_buf[chunksize] = '\0';
@@ -211,7 +211,7 @@ ft_t ft;
 	
 	if (strncmp(buf,"MDAT",4) != 0) 
 	{
-	    st_fail("MAUD: MDAT chunk not found");
+	    st_fail_errno(ft,ST_EFMT,"MAUD: MDAT chunk not found");
 	    return(ST_EOF);
 	}
 	st_readdw(ft, &(p->nsamples));
@@ -265,12 +265,12 @@ ft_t ft;
 	/* If you have to seek around the output file */
 	if (! ft->seekable) 
 	{
-	    st_fail("Output .maud file must be a file, not a pipe");
+	    st_fail_errno(ft,ST_EOF,"Output .maud file must be a file, not a pipe");
 	    return (ST_EOF);
 	}
 	
 	if (ft->info.channels != 1 && ft->info.channels != 2) {
-		st_fail("MAUD: unsupported number of channels, unable to store");
+		st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels, unable to store");
 		return(ST_EOF);
 	}
 	if (ft->info.size == ST_SIZE_WORD) ft->info.encoding = ST_ENCODING_SIGN2;
@@ -311,7 +311,7 @@ ft_t ft;
 	
 	if (fseek(ft->fp, 0L, 0) != 0) 
 	{
-	    st_fail("can't rewind output file to rewrite MAUD header");
+	    st_fail_errno(ft,errno,"can't rewind output file to rewrite MAUD header");
 	    return(ST_EOF);
 	}
 	
