@@ -31,7 +31,41 @@
 
 #define MAXWSPEED 1
 
-static void rawdefaults(P1(ft_t ft));
+#ifndef MIN
+#define MIN(a,b) ((a)<(b)?(a):(b))
+#endif
+
+static void rawdefaults(ft_t ft);
+
+
+int st_rawseek(ft,offset) 
+ft_t ft;
+LONG offset;
+{
+	int sample_size = 0;
+
+	switch(ft->info.size) {
+		case ST_SIZE_BYTE:
+			sample_size = 1;
+		        break;
+		case ST_SIZE_WORD:
+			sample_size = sizeof(short);
+		        break;
+		case ST_SIZE_DWORD:
+			sample_size = sizeof(LONG);
+		        break;
+		case ST_SIZE_FLOAT:
+			sample_size = sizeof(float);
+		        break;
+		default:
+			st_fail_errno(ft,ST_ENOTSUP,"Can't seek this data size");
+			return(ft->st_errno);
+	}
+
+	ft->st_errno = st_seek(ft,offset*sample_size,SEEK_SET);
+
+	return(ft->st_errno);
+}
 
 int st_rawstartread(ft) 
 ft_t ft;
@@ -39,7 +73,7 @@ ft_t ft;
 	ft->file.buf = malloc(BUFSIZ);
 	if (!ft->file.buf)
 	{
-	    st_fail("Unable to alloc resources");
+	    st_fail_errno(ft,ST_ENOMEM,"Unable to alloc resources");
 	    return(ST_EOF);
 	}
 	ft->file.size = BUFSIZ;
@@ -56,7 +90,7 @@ ft_t ft;
 	ft->file.buf = malloc(BUFSIZ);
 	if (!ft->file.buf)
 	{
-	    st_fail("Unable to alloc resources");
+	    st_fail_errno(ft,ST_ENOMEM,"Unable to alloc resources");
 	    return(ST_EOF);
 	}
 	ft->file.size = BUFSIZ;
@@ -66,30 +100,6 @@ ft_t ft;
 	return(ST_SUCCESS);
 }
 
-/* Read raw file data, and convert it to */
-/* the sox internal signed long format. */
-
-static unsigned char blockgetc(ft)
-ft_t ft;
-{
-	char rval;
-
-	if (ft->file.pos == ft->file.count)
-	{
-		if (ft->file.eof)
-			return(0);
-		ft->file.count = fread(ft->file.buf, 1, ft->file.size, ft->fp);
-		ft->file.pos = 0;
-		if (ft->file.count == 0)
-		{
-			ft->file.eof = 1;
-			return(0);
-		}
-	}
-	rval = *(ft->file.buf + ft->file.pos++);
-	return (rval);
-}
-
 /* Util to reverse the n chars starting at p. */
 /* FIXME: Move to misc.c */
 static void swapn(p, n)
@@ -107,178 +117,295 @@ int n;
 	}
 }
 
-/* Reads a block of 'n' characters and possibly byte swaps */
-static void blockr(p0, n, ft)
-ft_t ft;
-int n;
-void *p0;
+void st_ub_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
 {
-	int rest;
-	char *p;
-	p = p0;
-	
-	while ((rest = ft->file.count - ft->file.pos) < n)
-	{
-		if (ft->file.eof) {
-			memset(p,0,n);
-			return;
-		}
+    while (len)
+    {
+	unsigned char datum;
 
-		memmove(ft->file.buf, ft->file.buf+ft->file.pos, rest);
+	datum = *((unsigned char *)buf2);
+        buf2++;
+	datum ^= 0x80;
 
-		ft->file.pos = 0;
-		ft->file.count = rest;
-		ft->file.count += fread(ft->file.buf+rest, 1, ft->file.size-rest, ft->fp);
-		ft->file.eof = (ft->file.count < n);
-	}
-	memcpy(p, ft->file.buf + ft->file.pos, n);
-	ft->file.pos += n;
-	/* FIXME: For speed, there should be a version of this funciton
-	 * for WORDS, LONGS, and FLOATS.  This is because swapping
-	 * bytes is an expensive operation and should be done in batch
-	 * mode.
-	 */
-	if (ft->swap)
-		swapn(p,n);
+	*buf1++ = LEFT(datum,24);
+	len--;
+    }
 }
 
-static int blockr_sw(p0, n, ft)
-LONG *p0,n;
-ft_t ft;
+void st_sb_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
 {
-    LONG x, done;
-		short s;
+    while (len)
+    {
+	unsigned char datum;
 
-		for (done=0; done < n;) {
-			blockr(&s, sizeof(short), ft);
-			x = s;
-			if (ft->file.eof) break;
-			/* scale signed up to long's range */
-			*p0++ = LEFT(x, 16);
-			done++;
-		}
-	return done;
+	datum = *((unsigned char *)buf2);
+        buf2++;
+
+	*buf1++ = LEFT(datum,24);
+	len--;
+    }
+}
+
+void st_ulaw_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	unsigned char datum;
+
+	datum = *((unsigned char *)buf2);
+        buf2++;
+
+	*buf1++ = LEFT(st_ulaw_to_linear(datum),16);
+	len--;
+    }
+}
+
+void st_alaw_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	unsigned char datum;
+
+	datum = *((unsigned char *)buf2);
+        buf2++;
+
+	*buf1++ = LEFT(st_Alaw_to_linear(datum),16);
+	len--;
+    }
+}
+
+void st_uw_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	unsigned short datum;
+
+	datum = *((unsigned short *)buf2);
+        buf2++; buf2++;
+	if (swap)
+	    datum = st_swapw(datum);
+	datum ^= 0x8000;
+
+	*buf1++ = LEFT(datum,16);
+	len--;
+    }
+}
+
+void st_sw_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	unsigned short datum;
+
+	datum = *((unsigned short *)buf2);
+        buf2++; buf2++;
+	if (swap)
+	    datum = st_swapw(datum);
+
+	*buf1++ = LEFT(datum,16);
+	len--;
+    }
+}
+
+void st_ul_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	ULONG datum;
+
+	datum = *((ULONG *)buf2);
+        buf2++; buf2++; buf2++; buf2++;
+	if (swap)
+	    datum = st_swapl(datum);
+
+	datum ^= 0x80000000L;
+
+	*buf1++ = datum;
+	len--;
+    }
+}
+
+void st_sl_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	ULONG datum;
+
+	datum = *((ULONG *)buf2);
+        buf2++; buf2++; buf2++; buf2++;
+	if (swap)
+	    datum = st_swapl(datum);
+
+	*buf1++ = datum;
+	len--;
+    }
+}
+
+void st_f32_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	float datum;
+
+	datum = *((float *)buf2);
+        buf2++; buf2++; buf2++; buf2++;
+	if (swap)
+	    datum = st_swapf(datum);
+
+	*buf1++ = datum;
+	len--;
+    }
+}
+
+void st_f64_copy_buf(LONG *buf1, char *buf2, ULONG len, char swap)
+{
+    while (len)
+    {
+	double datum;
+
+	datum = *((double *)buf2);
+        buf2++; buf2++; buf2++; buf2++;
+        buf2++; buf2++; buf2++; buf2++;
+	if (swap)
+	    datum = st_swapd(datum);
+
+	*buf1++ = datum;
+	len--;
+    }
+}
+
+/* Reads a buffer of different data types into SoX's internal buffer
+ * format.
+ */
+/* FIXME:  This function adds buffering on top of stdio's buffering.
+ * Mixing st_readbuf's and freads or fgetc or even SoX's other util
+ * functions will cause a loss of data!  Need to have sox implement
+ * a consistent buffering protocol.
+ */
+ULONG st_readbuf(LONG *p, int n, int size, int encoding, ft_t ft)
+{
+    ULONG len, done = 0;
+    void (*copy_buf)(LONG *, char *, ULONG, char) = 0;
+    int i;
+
+    switch(size) {
+	case ST_SIZE_BYTE:
+	    switch(encoding)
+	    {
+		case ST_ENCODING_SIGN2:
+		    copy_buf = st_sb_copy_buf;
+		    break;
+		case ST_ENCODING_UNSIGNED:
+		    copy_buf = st_ub_copy_buf;
+		    break;
+		case ST_ENCODING_ULAW:
+		    copy_buf = st_ulaw_copy_buf;
+		    break;
+		case ST_ENCODING_ALAW:
+		    copy_buf = st_alaw_copy_buf;
+		    break;
+		default:
+		    st_fail_errno(ft,ST_EFMT,"Do not support this encoding for this data size.");
+		    return(0);
+	    }
+	    break;
+
+	case ST_SIZE_WORD:
+	    switch(encoding)
+	    {
+		case ST_ENCODING_SIGN2:
+		    copy_buf = st_sw_copy_buf;
+		    break;
+		case ST_ENCODING_UNSIGNED:
+		    copy_buf = st_uw_copy_buf;
+		    break;
+		default:
+		    st_fail_errno(ft,ST_EFMT,"Do not support this encoding for this data size.");
+		    return(0);
+	    }
+	    break;
+
+	case ST_SIZE_DWORD:
+	    switch(encoding)
+	    {
+		case ST_ENCODING_SIGN2:
+		    copy_buf = st_sl_copy_buf;
+		    break;
+		case ST_ENCODING_UNSIGNED:
+		    copy_buf = st_ul_copy_buf;
+		    break;
+		default:
+		    st_fail_errno(ft,ST_EFMT,"Do not support this encoding for this data size.");
+		    return(0);
+	    }
+	    break;
+
+	case ST_SIZE_FLOAT:
+	    copy_buf = st_f32_copy_buf;
+	    /* Hack hack... Encoding should be FLOAT, not the size */
+	    size = 4;
+	    break;
+
+	case ST_SIZE_DOUBLE:
+	    copy_buf = st_f64_copy_buf;
+	    /* Hack hack... Encoding should be FLOAT, not the size */
+	    size = 8;
+	    break;
+
+	default:
+	    st_fail_errno(ft,ST_EFMT,"Do not support this data size for this handler");
+	    return (0);
+    }
+
+    
+    len = MIN(n,(ft->file.count-ft->file.pos)/size);
+    if (len)
+    {
+	copy_buf(p + done, ft->file.buf + ft->file.pos, len, ft->swap);
+        ft->file.pos += (len*size);
+	done += len;
+    }
+
+    while (done < n)
+    {
+	/* See if there is not enough data in buffer for any more reads.
+	 * If not then shift any remaining data down to the beginning
+	 * and attempt to fill up the rest of the buffer.
+	 */
+	if (!ft->file.eof && ft->file.pos >= (ft->file.count-size+1))
+	{
+	    for (i = 0; i < (ft->file.count-ft->file.pos); i++)
+		ft->file.buf[i] = ft->file.buf[ft->file.pos+i];
+
+	    i = ft->file.count-ft->file.pos;
+	    ft->file.pos = 0;
+
+	    ft->file.count = fread(ft->file.buf+i, 1, ft->file.size-i, ft->fp) ;
+	    if (ft->file.count == 0)
+	    {
+		ft->file.eof = 1;
+	    }
+	    ft->file.count += i;
+	}
+
+        len = MIN(n - done,(ft->file.count-ft->file.pos)/size);
+        if (len)
+        {
+	    copy_buf(p + done, ft->file.buf + ft->file.pos, len, ft->swap);
+            ft->file.pos += (len*size);
+	    done += len;
+        }
+	if (ft->file.eof)
+	    break;
+    }
+    return done;
 }
 
 LONG st_rawread(ft, buf, nsamp) 
 ft_t ft;
 LONG *buf, nsamp;
 {
-	int done = 0;
-
-	switch(ft->info.size) {
-		case ST_SIZE_BYTE:
-		    switch(ft->info.encoding)
-		    {
-			case ST_ENCODING_SIGN2:
-				while(done < nsamp) {
-					LONG datum;
-					datum = blockgetc(ft);
-					if (ft->file.eof)
-						return done;
-					/* scale signed up to long's range */
-					*buf++ = LEFT(datum, 24);
-					done++;
-				}
-				return done;
-			case ST_ENCODING_UNSIGNED:
-				while(done < nsamp) {
-					LONG datum;
-					datum = blockgetc(ft);
-					if (ft->file.eof)
-						return done;
-					/* Convert to signed */
-					datum ^= 128;
-					/* scale signed up to long's range */
-					*buf++ = LEFT(datum, 24);
-					done++;
-				}
-				return done;
-			case ST_ENCODING_ULAW:
-				while(done < nsamp) {
-					LONG datum;
-					datum = blockgetc(ft);
-					if (ft->file.eof)
-						return done;
-					datum = st_ulaw_to_linear(datum);
-					/* scale signed up to long's range */
-					*buf++ = LEFT(datum, 16);
-					done++;
-				}
-				return done;
-			case ST_ENCODING_ALAW:
-				while(done < nsamp) {
-					LONG datum;
-					datum = blockgetc(ft);
-					if (ft->file.eof)
-						return done;
-					datum = st_Alaw_to_linear(datum);
-					/* scale signed up to long's range */
-					*buf++ = LEFT(datum, 16);
-					done++;
-				}
-				return done;
-		    }
-		    break;
-		case ST_SIZE_WORD:
-		    switch(ft->info.encoding)
-		    {
-			case ST_ENCODING_SIGN2:
-				return blockr_sw(buf, nsamp, ft);
-			case ST_ENCODING_UNSIGNED:
-				while(done < nsamp) {
-					LONG x;
-					unsigned short s;
-					blockr(&s, sizeof(short), ft);
-					x = s;
-					if (ft->file.eof)
-						return done;
-					/* Convert to signed */
-					x ^= 0x8000;
-					/* scale signed up to long's range */
-					*buf++ = LEFT(x, 16);
-					done++;
-				}
-				return done;
-			case ST_ENCODING_ULAW:
-				st_fail("No U-Law support for shorts");
-				return 0;
-			case ST_ENCODING_ALAW:
-				st_fail("No A-Law support for shorts");
-				return 0;
-		    }
-		    break;
-		case ST_SIZE_DWORD:
-		    switch(ft->info.encoding)
-		    {
-			case ST_ENCODING_SIGN2:
-				while(done < nsamp) {
-					blockr(buf, sizeof(LONG), ft);
-					if (ft->file.eof)
-						return done;
-					/* scale signed up to long's range */
-					buf++;
-					done++;
-				}
-				return done;
-		    }
-		    break;
-		case ST_SIZE_FLOAT:
-			while(done < nsamp) {
-				float f;
-				blockr(&f, sizeof(float), ft);
-				if (ft->file.eof)
-					return done;
-				*buf++ = f * 0x10000; /* hmm... */
-				done++;
-			}
-			return done;
-		default:
-			break;
-	}
-	st_fail("Sorry, don't have code to read %s, %s",
-		st_encodings_str[ft->info.encoding], st_sizes_str[ft->info.size]);
-	return(0);
+    return st_readbuf(buf, nsamp, ft->info.size, ft->info.encoding, ft);
 }
 
 int st_rawstopread(ft)
@@ -294,7 +421,7 @@ ft_t ft;
 {
 	if (fwrite(ft->file.buf, 1, ft->file.pos, ft->fp) != ft->file.pos)
 	{
-		st_fail("Error writing data to file");
+		st_fail_errno(ft,errno,"Error writing data to file");
 	}
 	ft->file.pos = 0;
 }
@@ -452,10 +579,10 @@ LONG *buf, nsamp;
 				}
 				return done;
 			case ST_ENCODING_ULAW:
-				st_fail("No U-Law support for shorts");
+				st_fail_errno(ft,ST_EFMT,"No U-Law support for shorts");
 				return 0;
 			case ST_ENCODING_ALAW:
-				st_fail("No A-Law support for shorts");
+				st_fail_errno(ft,ST_EFMT,"No A-Law support for shorts");
 				return 0;
 		    }
 		    break;
@@ -484,7 +611,8 @@ LONG *buf, nsamp;
 		default:
 			break;
 	}
-	st_fail("Sorry, don't have code to write %s, %s",
+	/* Possible overflow */
+	st_fail_errno(ft,ST_EFMT,"Sorry, don't have code to write %s, %s",
 		st_encodings_str[ft->info.encoding], st_sizes_str[ft->info.size]);
 	return 0;
 }
