@@ -90,7 +90,7 @@ int st_txwstartread(ft)
   /* If you need to seek around the input file. */
   if (! ft->seekable)
   {
-    st_fail("txw input file must be a file, not a pipe");
+    st_fail_errno(ft,ST_EOF,"txw input file must be a file, not a pipe");
     return(ST_EOF);
   }
 
@@ -127,7 +127,7 @@ int st_txwstartread(ft)
   st_report("Found header filetype %s",filetype);
   if(strcmp(filetype,"LM8953"))
   {
-    st_fail("Invalid filetype ID in input file header, != LM8953");
+    st_fail_errno(ft,ST_EHDR,"Invalid filetype ID in input file header, != LM8953");
     return(ST_EOF);
   }
   /*
@@ -136,13 +136,13 @@ int st_txwstartread(ft)
 
   switch( sample_rate ) {
   case 1:
-    ft->info.rate = 33000;
+    ft->info.rate = 33333;
     break;
   case 2:
     ft->info.rate = 50000;
     break;
   case 3:
-    ft->info.rate = 16000;
+    ft->info.rate = 16667;
     break;
   default:
     blewIt = 1;
@@ -150,7 +150,7 @@ int st_txwstartread(ft)
     case 0x06:
       if ( (gunk[5] & 0xFE) == 0x52 ) {
 	blewIt = 0;
-	ft->info.rate = 33000;
+	ft->info.rate = 33333;
       }
       break;
     case 0x10:
@@ -162,13 +162,13 @@ int st_txwstartread(ft)
     case 0xF6:
       if ( (gunk[5] & 0xFE) == 0x52 ) {
 	blewIt = 0;
-	ft->info.rate = 16000;
+	ft->info.rate = 16667;
       }
       break;
     }
     if ( blewIt ) {
       st_report("Invalid sample rate identifier found %d", (int)sample_rate);
-      ft->info.rate = 33000;
+      ft->info.rate = 33333;
     }
   }
   st_report("Sample rate = %ld",ft->info.rate);
@@ -267,7 +267,7 @@ ft_t ft;
   /* If you have to seek around the output file */
   if (! ft->seekable)
   {
-      st_fail("Output .txw file must be a file, not a pipe");
+      st_fail_errno(ft,ST_EOF,"Output .txw file must be a file, not a pipe");
       return(ST_EOF);
   }
 
@@ -331,7 +331,7 @@ ft_t ft;
     else                            WH.sample_rate = 2;
     
     if (tx16w_len >= TXMAXLEN) {
-        fprintf(stderr,"Sound too large for TX16W. Truncating, Loop Off\n");
+        st_warn("Sound too large for TX16W. Truncating, Loop Off\n");
         AttackLength       = TXMAXLEN/2;
         LoopLength         = TXMAXLEN/2;
     }
